﻿Type=Service
Version=2.71
@EndOfDesignText@
#Region  Service Attributes 
	#StartAtBoot: False
#End Region

Sub Process_Globals

	Dim GooglePlayServicesHelper1 As GooglePlayServicesHelper
	Dim LocationClient1 As LocationClient
	Dim LocationListener1 As LocationListener
	Dim LocationStatusCodes1 As LocationStatusCodes
	Dim NotificationId As Int=1
	Dim OnAddGeofencesResultListener1 As OnAddGeofencesResultListener

End Sub

Sub Service_Create

	LocationClient1.Initialize("LocationClient1")
	
	Dim Notification1 As Notification
	Notification1.Initialize
	Notification1.AutoCancel=False
	Notification1.Icon="icon"	'	uses the application icon
	Notification1.Light=False
	Notification1.OnGoingEvent=True
	Notification1.SetInfo("LocationAPI demo", "LocationManager is running", Main)
	Notification1.Sound=False
	Notification1.Vibrate=False
	
	Service.StartForeground(NotificationId, Notification1)
	
End Sub

Sub Service_Start (StartingIntent As Intent)

	If LocationClient1.IsConnected Then
		CallSub2(Main, "UpdateStatus", "Waiting for location updates")
	Else
		Dim ServiceStatus As Int=GooglePlayServicesHelper1.IsGooglePlayServicesAvailable
		Select ServiceStatus
			Case GooglePlayServicesHelper1.SUCCESS
				Log("GooglePlayServicesHelper IsGooglePlayServicesAvailable service available")
				CallSub2(Main, "UpdateStatus", "LocationClient connection")
				LocationClient1.Connect
			Case Else
				Log("GooglePlayServicesHelper IsGooglePlayServicesAvailable returned: "&ServiceStatus)
				CallSub2(Main, "UpdateStatus", "Google Play Services unavailable")
				StopService("")
		End Select
	End If
	
End Sub

Sub Service_Destroy
	'	i don't think we actually need to remove the listener and geofences
	'	i think calling Disconnect is all we need to do
	'	need to check and test...
	
	If LocationListener1.IsInitialized Then
		LocationClient1.RemoveLocationUpdates(LocationListener1)
	End If
	
	If OnAddGeofencesResultListener1.IsInitialized Then
		Dim OnRemoveGeofencesResultListener1 As OnRemoveGeofencesResultListener
		OnRemoveGeofencesResultListener1.Initialize("OnRemoveGeofencesResultListener1")
		Dim GeofenceIds As List
		GeofenceIds.Initialize2(Array As String("MyGeofence"))
		LocationClient1.RemoveGeofences(GeofenceIds, OnRemoveGeofencesResultListener1)
	End If
	
	If LocationClient1.IsConnected Then 
		LocationClient1.Disconnect
	End If
	
End Sub

Sub LocationClient1_Connected
	Log("LocationClient1_Connected")
	
	'	not sure if this is a good idea, to pass the last known location to the LocationChanged Sub
	Dim LastLocation As Location=LocationClient1.GetLastLocation
	If LastLocation.IsInitialized Then
		Log("Manually calling LocationListener1_LocationChanged")
		LocationListener1_LocationChanged(LastLocation)
	End If
	
	If Main.PreferenceManager1.GetBoolean("LocationRequestEnabled") Then
		Dim LocationRequest1 As LocationRequest
		LocationRequest1.Initialize
		
		LocationRequest1.SetExpirationDuration(Main.PreferenceManager1.GetString("ExpirationDuration"))
		LocationRequest1.SetFastestInterval(Main.PreferenceManager1.GetString("FastestInterval"))
		LocationRequest1.SetInterval(Main.PreferenceManager1.GetString("Interval"))
		LocationRequest1.SetNumUpdates(Main.PreferenceManager1.GetString("NumUpdates"))
		LocationRequest1.SetPriority(Main.PreferenceManager1.GetString("Priority"))
		LocationRequest1.SetSmallestDisplacement(Main.PreferenceManager1.GetString("SmallestDisplacement"))
		
		LocationListener1.Initialize("LocationListener1")
		LocationClient1.RequestLocationUpdates(LocationRequest1, LocationListener1)
	End If
	
	If Main.PreferenceManager1.GetBoolean("GeofenceEnabled") Then
		Dim Geofence1 As Geofence
		
		Dim GeofenceBuilder1 As GeofenceBuilder
		GeofenceBuilder1.Initialize
		GeofenceBuilder1.SetCircularRegion(52.756096, 0.397613, 20)
		GeofenceBuilder1.SetExpirationDuration(Geofence1.NEVER_EXPIRE)
		GeofenceBuilder1.SetRequestId("MyGeofence")
		GeofenceBuilder1.SetTransitionTypes(Bit.OR(Geofence1.GEOFENCE_TRANSITION_ENTER, Geofence1.GEOFENCE_TRANSITION_EXIT))
		
		Geofence1=GeofenceBuilder1.Build
		
		Dim Geofences As List
		Geofences.Initialize
		Geofences.Add(Geofence1)
		
		Dim Intent1 As Intent
		Intent1.Initialize("", "")
		Intent1.SetComponent("uk.co.martinpearman.b4a.locationapidemo/.intenthandler")
		
		Dim PendingIntent1 As PendingIntent
		PendingIntent1.InitializeService(0, Intent1, PendingIntent1.FLAG_UPDATE_CURRENT)
		
		OnAddGeofencesResultListener1.Initialize("OnAddGeofencesResultListener1")
		
		LocationClient1.AddGeofences(Geofences, PendingIntent1, OnAddGeofencesResultListener1)
	End If
	
	CallSub2(Main, "UpdateStatus", "Waiting for location updates")
	
End Sub

Sub LocationClient1_ConnectionFailed(ErrorCode As Int)
	Log("LocationClient1_ConnectionFailed ErrorCode="&ErrorCode)
	CallSub2(Main, "UpdateStatus", "LocationClient failed to connect")
	StopService("")
End Sub

Sub LocationClient1_Disconnected
	Log("LocationClient1_Disconnected")
	CallSub2(Main, "UpdateStatus", "LocationClient disconnected")
	StopService("")
End Sub

'	** the GPS library must be enabled to work with Location objects **
Sub LocationListener1_LocationChanged(Location1 As Location)
	Log("LocationListener1_LocationChanged")
	Main.Locations.Add(Location1)
	CallSub(Main, "DisplayLocations")
End Sub

Sub OnAddGeofencesResultListener1_AddGeofencesResult(StatusCode As Int, GeofenceRequestIds() As String)
	Log("OnAddGeofencesResultListener1_AddGeofencesResult")
	
	Dim i As Int
	For i=0 To GeofenceRequestIds.Length-1
		Select StatusCode
			Case LocationStatusCodes1.ERROR
				Log("ERROR "&GeofenceRequestIds(i))
			Case LocationStatusCodes1.GEOFENCE_NOT_AVAILABLE
				Log("GEOFENCE_NOT_AVAILABLE "&GeofenceRequestIds(i))
			Case LocationStatusCodes1.GEOFENCE_TOO_MANY_GEOFENCES
				Log("GEOFENCE_TOO_MANY_GEOFENCES "&GeofenceRequestIds(i))
			Case LocationStatusCodes1.GEOFENCE_TOO_MANY_PENDING_INTENTS
				Log("GEOFENCE_TOO_MANY_PENDING_INTENTS "&GeofenceRequestIds(i))
			Case LocationStatusCodes1.SUCCESS
				Log("SUCCESS "&GeofenceRequestIds(i))
				CallSub2(Main, "UpdateStatus", "Geofence added")
		End Select
	Next
	
End Sub

Sub OnRemoveGeofencesResultListener1_RemoveGeofencesByRequestIdsResult(StatusCode As Int, GeofenceRequestIds() As String)
	Log("OnRemoveGeofencesResultListener1_RemoveGeofencesByRequestIdsResult")
End Sub